<?php

namespace Drupal\safedelete\Controller;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\node\Entity\NodeType;
use Drupal\views\Views;
use Drupal\Core\Url;
use Drupal\safedelete\AdminHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\file\Entity\File;
use Drupal\Core\Datetime;


class SearchingOrphanedPageController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;


  /**
   * SearchingOrphanedPageController constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    //parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
  }

  public static function create(ContainerInterface $container) {
    $database = $container->get('database');
    return new static($database);
  }

  /**
   * Display the markup.
   *
   * @return array
   */
  public function content() {
    $viewreport = FALSE;
    $rountingname = \Drupal::routeMatch()->getRouteName();
    if (\Drupal::routeMatch()->getRouteName() == 'safedelete_orphanedpages.content') {
      if (\Drupal::currentUser()->isAuthenticated()) {
        // Was doing this to eliminate admin rendering but template changes should have stripped most of that out.
        // Uncomment the two lines below if we want to force a logout.
        //$session_manager = \Drupal::service('session_manager');
        //$session_manager->delete(\Drupal::currentUser()->id());
      }
    }
    if (\Drupal::routeMatch()->getRouteName() == 'safedelete_vieworphanedpagesreport.admin_content') {
      $viewreport = TRUE;
    }
    $vars_array = [];
    $vars_array['config_url'] = Url::fromRoute('safedelete.settings')->toString();
    $vars_array['system_cron_url']  = Url::fromRoute('system.cron',['key' => \Drupal::state()->get('system.cron_key')], ['absolute' => TRUE])->toString();
    return [
      '#theme' => 'safedelete_orphanedpages',
      '#showconfigsettinglink' => $this->showconfigsettinglink(),
      '#orphanedpages_container' => $this->getOrphanedLandingPageOrBasicPage($viewreport),
      '#vars_array' => $vars_array
    ];
  }


  /**
   * get orphaned pages
   */
  public function getOrphanedLandingPageOrBasicPage ($viewreport = FALSE) {
    $reporteddate = '';
    $orphanedpages_results = [];
    $filedirectory = 'public://orphaned_pages_files/';

    if ($viewreport) { // read the report from file system
      $directoryexists = \Drupal::service('file_system')->prepareDirectory($filedirectory);
      if(!$directoryexists) {
        \Drupal::service('file_system')->mkdir($filedirectory);
      }
      else {
        $options['key'] = "name";
        $files = \Drupal::service('file_system')->scanDirectory($filedirectory, '/.json/');
        $filearraynid = array();
        foreach ($files as $file) {
          $nodeidstr = $file ->name;
          $nodeid = (int) $nodeidstr;
          $filesarraynid[$nodeid] = $file;
        }
        if(isset($filesarraynid)) {
          ksort($filesarraynid);
          $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
          foreach ($filesarraynid as $key => $file) {
            $uri = $file->uri;
            $buffer =  file_get_contents($uri);
            $jsondata = json_decode($buffer);
            $nid = $jsondata->nid;
            $reporteddate = $jsondata->reporteddate;
            $orphanedpages_results[$nid]['nid'] =  $nid;
            $orphanedpages_results[$nid]['type'] = $jsondata->type;
            $orphanedpages_results[$nid]['href'] =  AdminHelper::getUrlForNode($nid, $langcode);
            $orphanedpages_results[$nid]['title'] = $jsondata->title;
            $orphanedpages_results[$nid]['reason'] = $jsondata->reason;
            $orphanedpages_results[$nid]['reporteddate'] = $reporteddate;
          }
        }
      }
    }
    else {
      // generate report
      AdminHelper::listOrphanedNodes($reporteddate, $orphanedpages_results, $filedirectory, $this->database);
    }
    $finalresult = array (
      'reporteddate' =>$reporteddate,
      'orphanedpages_results' => $orphanedpages_results
    );
    return $finalresult;
  }


  public function showconfigsettinglink() {
    // Check if the user has the "safedelete administration" permission.
    $user = \Drupal::currentUser();
    $has_permission = $user->hasPermission('safedelete administration');
    return $has_permission;
  }
}
