<?php

namespace Drupal\safedelete\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use \Drupal\node\Entity\NodeType;
use Drupal\safedelete\AdminHelper;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\Core\Database\Connection;

/**
 * Configure safedelete settings for this site.
 */
class SafedeleteSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'safedelete_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'safedelete.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('safedelete.settings');
    $node_types = NodeType::loadMultiple();
    if (empty($node_types)) {
      return NULL;
    }
    $options = [];
    $first_bundle = '';
    foreach ($node_types as $node_type_test => $type_test) {
      $first_bundle = $node_type_test;
      break;
    }
    foreach ($node_types as $node_type => $type) {
      $options[$node_type] = $type->get('name');
    }

    $form['bundle'] = [
      '#title' => t('Bundle'),
      '#type' => 'checkboxes',
      '#description' => t('Check the content types that you wish to add restriction on deletion'),
      '#options' => $options,
      '#default_value' => (!empty($config->get('bundle'))) ? $config->get('bundle') : [$first_bundle]
    ];

    $form['safearchive'] = [
      '#title' => t('Safe archive'),
      '#type' => 'checkbox',
      '#description' => t('This option will enable safe archive validation (future option, not yet implemented)'),
      '#default_value' => (!empty($config->get('safearchive'))) ? $config->get('safearchive') : TRUE,
      '#disabled' => TRUE,
    ];

    $form['delete_button'] = [
      '#title' => t('Show delete button'),
      '#type' => 'checkbox',
      '#description' => t('This option will show delete button in the file delete form page, even when the file is associated with entities'),
      '#default_value' => (!empty($config->get('delete_button'))) ? $config->get('delete_button') : FALSE
    ];

    $form['limit'] = [
      '#title' => t('Number of entities list to show in node delete form'),
      '#type' => 'textfield',
      '#description' => t("Number of entities list to show in node delete form"),
      '#size' => 2,
      '#default_value' => (!empty($config->get('limit'))) ? $config->get('limit') : FALSE
    ];

    /* the definition of the orphaned page */
    $form['orphanedpagedefinition'] = [
      '#title' => t('The definition of the orphaned pages'),
      '#type' => 'checkboxes',
      '#description' => t('Check the content types that you wish to consider as the orphaned pages'),
      '#options' => $options,
      '#default_value' => (!empty($config->get('orphanedpagedefinition'))) ? $config->get('orphanedpagedefinition') : [$first_bundle]
    ];

    /* the without menu link definition of the orphaned page */
    $form['orphanedpagewithoutmenulink'] = [
      '#title' => t('Choose one type to include menu link checking in the orphaned pages logic'),
      '#type' => 'radios',
      '#description' => t('Check the content type that you wish to consider as the orphaned pages even if it is only a menu link referencing the content'),
      '#options' => $options,
      '#default_value' => (!empty($config->get('orphanedpagewithoutmenulink'))) ? $config->get('orphanedpagewithoutmenulink') : $first_bundle
    ];

    $form['exceptionofnodes'] = [
      '#title' => t('Please input the node ids into the exception list of the orphaned pages'),
      '#type' => 'textfield',
      '#size' => 160,
      '#maxlength' => 326,
      '#description' => t('Enter node ids and separate them by comma'),
      '#default_value' => $config->get('exceptionofnodes'),
    ];

    // Buttons are only required for testing and debugging reasons.
    $orphdescription = '<p>' . $this->t('The section will run checking orphaned nodes proccess.') ;
    $form['clear'] = [
      '#type' => 'details',
      '#title' => $this->t('Maintenance'),
      '#description' => $orphdescription,
      '#open' => FALSE,
    ];
    $form['clear']['orphanedpage_check'] = [
      '#type' => 'submit',
      '#value' => $this->t('Check orphaned nodes'),
      '#submit' => ['::submitForm', '::submitOrphanedpageCheck'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('safedelete.settings')
      ->set('bundle', $form_state->getValue('bundle'))
      ->set('delete_button', $form_state->getValue('delete_button'))
      ->set('exceptionofnodes', $form_state->getValue('exceptionofnodes'))
      ->set('limit', $form_state->getValue('limit'))
      ->set('orphanedpagedefinition', $form_state->getValue('orphanedpagedefinition'))
      ->set('orphanedpagewithoutmenulink', $form_state->getValue('orphanedpagewithoutmenulink'))
      ->set('safearchive', $form_state->getValue('safearchive'))
      ->save();

    parent::submitForm($form, $form_state);
  }


  /**
   * Check all links.
   */
  public function submitOrphanedpageCheck(array &$form, FormStateInterface $form_state) {
    $batch = [];
    $batch = $this->generateBatch();
    batch_set($batch);
  }

 /**
   * Create a batch
   */
  public function generateBatch() {
    $reporteddate = '';
    $orphanedpages_results = [];
    $filedirectory = 'public://orphaned_pages_files/';
    $database = \Drupal::database();
    $node_types = NodeType::loadMultiple();
    if (empty($node_types)) {
      return NULL;
    }

    $first_bundle = ''; // A safe default value is required.
    foreach ($node_types as $node_type_test => $type_test) {
      $first_bundle = $node_type_test; // A safe default value.
      break;
    }

    $exceptidsconfig = \Drupal::config('safedelete.settings')->get('exceptionofnodes');
    $exceptionarray = explode(',', $exceptidsconfig);
    if (empty($exceptionarray)) {
      $exceptionarray = [];
    }
    $super_type = \Drupal::config('safedelete.settings')->get('orphanedpagewithoutmenulink');
    if (empty($super_type)) {
      // Use the safe default value.
      $super_type = $first_bundle;
    }
    $defconfig = \Drupal::config('safedelete.settings')->get('orphanedpagedefinition');
    if (empty($defconfig)) {
      // Use the safe default value.
      $defconfig = [];
      $defconfig[] = $first_bundle;
    }
    $query = \Drupal::entityQuery('node');
    $query->condition('type', $defconfig, 'IN');
    $nodeids =  $query->sort('nid', 'ASC')->execute();
    // create an index array for Patch API
    $arraryofNodeIds = [];
    $indx = 0;
    foreach ($nodeids as $nid ) {
      $arraryofNodeIds[$indx] =  $nid;
      $indx = $indx + 1;
    }

    $arrayoftypes = [];
    $count = 0;
    foreach ( $defconfig as $type) {
      if ( strlen($type) > 1 ) {
        $count = $count +1;
        $arrayoftypes[$count] = $type;
      }
    }
    $directoryexists = \Drupal::service('file_system')->prepareDirectory($filedirectory);
    if(!$directoryexists) {
      \Drupal::service('file_system')->mkdir($filedirectory);
    }
    else {
      // clean up old json files before new json files are generated
      $files = \Drupal::service('file_system')->scanDirectory($filedirectory, '/.json/');
      $filearraynid = array();
      foreach ($files as $file) {
        $fileuri = $file ->uri;
        \Drupal::service('file_system')->deleteRecursive($fileuri);
      }
    }

    $num_operations = count($arraryofNodeIds);
    $operations = [];
    //$i = 0;
    for ($i = 0; $i < $num_operations; $i++) {
     if (isset ($arraryofNodeIds[$i])) {
        $nid = (int) $arraryofNodeIds[$i];
        $operations[] = [
          'batch_ProcessNodes_Op',
          [
            $i + 1,
            $this->t('(Operation @operation)', ['@operation' => $i]),
            $nid,
            $filedirectory,
            $exceptionarray,
            $super_type,
          ],
        ];
      }
    }
    $batch = [
      'title' => $this->t('Validating orphaned nodes process will run @num operations', ['@num' => $num_operations]),
      'operations' => $operations,
      'finished' => 'batch_ProcessNodes_finished',
    ];
    return $batch;
  }
}
